@@ -1,11 +1,11 @@
-/*
+/****************************************************************************
  *
- * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2012
+ * Copyright (c) 1996-2020 Vladi Belperchinov-Shabanski "Cade" 
  * http://cade.datamax.bg/  <cade@biscom.net> <cade@bis.bg> <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
- */
+ ****************************************************************************/
 
 
 #include "vfuarc.h"
@@ -27,7 +27,7 @@ void vfu_read_archive_files( int a_recursive )
     archive_path = ""; /* cannot have path when recursing archive */
 
   VString s;
-  s = "/usr/lib/vfu/rx_auto ";
+  s = "/usr/libexec/vfu/rx_auto ";
   s += ( a_recursive ) ? "v" : "l";
   s += " '" + archive_name + "' ";
   s += " '" + archive_path + "' ";
@@ -124,10 +124,10 @@ void vfu_browse_archive_file()
     }
   chdir( tmpdir );
 
-  VString fn = files_list[FLI]->full_name();
+  VString fn = files_list_get(FLI)->full_name();
 
   VString s;
-  s = "/usr/lib/vfu/rx_auto x \"";
+  s = "/usr/libexec/vfu/rx_auto x \"";
   s += work_path;
   s += archive_name;
   s += "\" ";
@@ -157,10 +157,10 @@ void vfu_user_external_archive_exec( VString &shell_line  )
     }
   chdir( tmpdir );
 
-  VString fn = files_list[FLI]->full_name();
+  VString fn = files_list_get(FLI)->full_name();
 
   VString s;
-  s = "/usr/lib/vfu/rx_auto x \"";
+  s = "/usr/libexec/vfu/rx_auto x \"";
   s += work_path;
   s += archive_name;
   s += "\" ";
@@ -190,7 +190,7 @@ void vfu_extract_files( int one )
   if ( one == 0 )
     sprintf( t, "EXTRACT SELECTION to: " );
   else
-    sprintf( t, "EXTRACT `%s' to:", files_list[FLI]->full_name() );
+    snprintf( t, sizeof(t), "EXTRACT `%s' to:", files_list_get(FLI)->full_name() );
 
   target = opt.last_copy_path[ CM_COPY ];
   if ( !vfu_get_dir_name( t, target ) ) return;
@@ -200,13 +200,13 @@ void vfu_extract_files( int one )
   VArray va;
 
   int z;
-  for( z = 0; z < files_count; z++ )
-    if ((files_list[z]->sel && one == 0) || (z == FLI && one != 0))
-      va.push( files_list[z]->full_name() );
+  for( z = 0; z < files_list_count(); z++ )
+    if ((files_list_get(z)->sel && one == 0) || (z == FLI && one != 0))
+      va.push( files_list_get(z)->full_name() );
 
   if (chdir(target))
     {
-    sprintf( t, "Cannot chdir to: %s", target.data() );
+    snprintf( t, sizeof(t), "Cannot chdir to: %s", target.data() );
     say1( t );
     say2errno();
     return;
@@ -215,14 +215,14 @@ void vfu_extract_files( int one )
   VString tmpfile = vfu_temp();
   if (va.fsave( tmpfile ))
     {
-    sprintf( t, "Error writing list file: %s", tmpfile.data() );
+    snprintf( t, sizeof(t), "Error writing list file: %s", tmpfile.data() );
     say1( t );
     return;
     }
   chmod( tmpfile, S_IRUSR|S_IWUSR );
 
   VString s;
-  s = "/usr/lib/vfu/rx_auto x \"";
+  s = "/usr/libexec/vfu/rx_auto x \"";
   s += work_path;
   s += archive_name;
   s += "\" @";
@@ -234,13 +234,13 @@ void vfu_extract_files( int one )
   if (unlink( tmpfile ))
     {
     /*
-    sprintf( t, "Cannot unlink/erase temp file: %s", tmpfile );
+    snprintf( t, sizeof(t), "Cannot unlink/erase temp file: %s", tmpfile );
     say2( t );
     */
     }
   if (chdir(work_path))
     {
-    sprintf( t, "Cannot chdir back to to: %s", work_path.data() );
+    snprintf( t, sizeof(t), "Cannot chdir back to to: %s", work_path.data() );
     say1( t );
     say2errno();
     return;
