@@ -1,11 +1,11 @@
-/*
+/****************************************************************************
  *
- * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2012
+ * Copyright (c) 1996-2020 Vladi Belperchinov-Shabanski "Cade" 
  * http://cade.datamax.bg/  <cade@biscom.net> <cade@bis.bg> <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
- */
+ ****************************************************************************/
 
 #include "vfu.h"
 #include "vfudir.h"
@@ -22,16 +22,14 @@
 **
 ****************************************************************************/
 
-const char *CM_DESC[] = { "COPY", "MOVE", "LINK" };
+const char *CM_DESC[] = { "COPY", "MOVE", "SYMLINK" };
 char *copy_buff = NULL;
 
 int ignore_copy_errors = 0; /* actually it is used for copy/move/erase */
 
 /* clipboard ***************************************************************/
 
-const char *CB_DESC[] = { "COPY", "MOVE", "SYMLINK" };
-
-VTrie Clipboard;
+VTrie clipboard;
 CopyInfo clipboard_copy_info;
 
 /****************************************************************************
@@ -40,7 +38,7 @@ CopyInfo clipboard_copy_info;
 **
 ****************************************************************************/
 
-fsize_t device_free_space( const char *target ) /* TODO: get bavail/bfree depending on current user */
+fsize_t device_free_space( const char *target )
 {
   struct statfs stafs;
   int res = statfs( str_file_path( target ), &stafs );
@@ -48,6 +46,14 @@ fsize_t device_free_space( const char *target ) /* TODO: get bavail/bfree depend
   return 0;
 }
 
+fsize_t device_avail_space( const char *target )
+{
+  struct statfs stafs;
+  int res = statfs( str_file_path( target ), &stafs );
+  if( res == 0 ) return ((fsize_t)(stafs.f_bsize)) * stafs.f_bavail;
+  return 0;
+}
+
 /*
  *  return 0 if src and dst are actually the same file
  */
@@ -127,24 +133,55 @@ int device_is_same( const char *src, const char *dst )
 
 void show_copy_pos( fsize_t a_fc, /* file under copy current pos */
                     fsize_t a_fa, /* file under copy all size */
+                    long    a_et, /* elapsed time for current file copy */
                     CopyInfo *copy_info ) /* totals info */
 {
-  char t[16];
+  char t[128];
 
   fsize_t c1 = a_fc;
   fsize_t a1 = a_fa;
   fsize_t c2 = copy_info->current_size;
   fsize_t a2 = copy_info->files_size;
 
+  long t1  = a_et;
+  long t2  = copy_info->elapsed_time;
+  long eta = ((t1+t2)*a2)/(c1+c2) - (t1+t2);
+
+  int eta_h = eta / ( 60*60 );
+  int eta_m = eta / 60;
+  int eta_s = eta % 60;
+  
+  int  eta_v = eta_m > 0 ? eta_m : eta_s;
+  char eta_c = eta_m > 0 ? eta_m : eta_s;
+  if( eta_h > 0 )
+    {
+    eta_v = eta_h;
+    eta_c = 'h';
+    }
+  else if( eta > 99 && eta_m > 0 )
+    {
+    eta_v = eta_m;
+    eta_c = 'm';
+    }
+  else
+    {
+    eta_v = eta;
+    eta_c = 's';
+    }  
+  
+  int speed = -1;
+  // if( t1 > 0 ) speed = ( c1 / ( 1024 * 1024 ) ) / t1; // current MiB/s
+  if( t1+t2 > 0 ) speed = ( (c1+c2) / ( 1024 * 1024 ) ) / (t1+t2); // total MiB/s
+
   ASSERT( a1 >= 0 && a2 >= 0 );
 
   if ( a1 < 1 ) a1 = 1;
   if ( a2 < 1 ) a2 = 1;
   if ( c1 == a1 ) /* hack, every single 100% each is not meaningfull really */
-    sprintf( t, "     %%%5.1f", (100.0*(c1+c2))/a2);
+    sprintf( t, "     %%%5.1f @%3dM/s%3d%c", (100.0*(c1+c2))/a2, speed, eta_v, eta_c );
   else
-    sprintf( t, "%5.1f%%%5.1f", (100.0*c1)/a1, (100.0*(c1+c2))/a2);
-  con_out( con_max_x() - 12, con_max_y(), t, cSTATUS2 );
+    sprintf( t, "%5.1f%%%5.1f @%3dM/s%3d%c", (100.0*c1)/a1, (100.0*(c1+c2))/a2, speed, eta_v, eta_c );
+  con_out( con_max_x() - 24, con_max_y(), t, cSTATUS2 );
 }
 
 /*###########################################################################*/
@@ -184,15 +221,17 @@ int over_if_exist( const char* src, const char *dst, CopyInfo* copy_info )
 
     time_str_compact( stat_src.st_mtime, sttime);
     str = file_st_size( &stat_src );
-    str_comma(str);
-    sprintf(t, "SRC: %s%c %11s%c %s", sttime, s_t, str.data(), s_s, src );
-    say1(t);
+    vfu_str_comma(str);
+    str_pad( str, 15 );
+    snprintf( t, sizeof(t), "SRC: %s%c %s%c %s", sttime, s_t, str.data(), s_s, src );
+    say1( t );
 
     time_str_compact(stat_dst.st_mtime, sttime);
     str = file_st_size( &stat_dst );
-    str_comma(str);
-    sprintf(t, "DST: %s%c %11s%c %s", sttime, s_t, str.data(), s_s, dst );
-    say2(t);
+    vfu_str_comma(str);
+    str_pad( str, 15 );
+    snprintf( t, sizeof(t), "DST: %s%c %s%c %s", sttime, s_t, str.data(), s_s, dst );
+    say2( t );
 
     vfu_beep();
     vfu_menu_box( "Overwrite", "Y Yes,N No,A Always overwrite,V Never overwrite,I If newer (MODIFY),W Always if newer (MODIFY),D View differences,  Abort (ESC)", -1 );
@@ -245,7 +284,8 @@ int vfu_copy_mode( const char* src, const char* dst )
 
   /* copy owner/group */
   if (opt.copy_keep_mode)
-    chown( dst, st.st_uid, st.st_gid );
+    if (chown( dst, st.st_uid, st.st_gid ))
+      say1( "Cannot change file mode" );
 
   return 0;
 }
@@ -295,7 +335,7 @@ int __vfu_file_copy( const char* src, const char* dst, CopyInfo* copy_info )
 
   if ( !copy_info->no_free_check && !copy_info->no_info )
     {
-    fsize_t dev_free = device_free_space( dst );
+    fsize_t dev_free = device_avail_space( dst );
     if (size > dev_free )
       {
       char t[128];
@@ -345,13 +385,21 @@ int __vfu_file_copy( const char* src, const char* dst, CopyInfo* copy_info )
 
   int aborted = 0;
 
+  
+  time_t timer_start   = time(NULL);
+  long   elapsed_time  = 0;
+  long   elapsed_break = 0;
+  
   do
     {
+    time_t timer_break = time(NULL);
     if ( vfu_break_op() )
       {
       aborted = 1;
       break;
       }
+    elapsed_break += time(NULL) - timer_break;
+      
     z = fread( copy_buff, 1, COPY_BUFFER_SIZE, f1 );
     if (z > 0) z = fwrite( copy_buff, 1, z, f2 );
     if (z == -1)
@@ -362,8 +410,9 @@ int __vfu_file_copy( const char* src, const char* dst, CopyInfo* copy_info )
       return 1;
       }
     cp += z;
-    ASSERT( cp <= size );
-    show_copy_pos( cp, size, copy_info );
+    //ASSERT( cp <= size );
+    elapsed_time = time(NULL) - timer_start - elapsed_break;
+    show_copy_pos( cp, size, elapsed_time, copy_info );
     }
   while ( z == COPY_BUFFER_SIZE );
 
@@ -376,12 +425,13 @@ int __vfu_file_copy( const char* src, const char* dst, CopyInfo* copy_info )
     if ( aborted ) return CR_ABORT;
     return 1;
     }
-  ASSERT( cp == size );
+  //ASSERT( cp == size );
 
   if ( vfu_copy_mode( src, dst ) ) return 1;
 
   copy_info->current_size += size;
-  show_copy_pos( 1, 1, copy_info );
+  copy_info->elapsed_time += elapsed_time;
+  show_copy_pos( 1, 1, 0, copy_info );
   return 0;
 }
 
@@ -393,7 +443,7 @@ int __vfu_file_move( const char* src, const char* dst, CopyInfo* copy_info )
 
   if ( device_is_same( src, dst ) == 0 )
     { /* same device */
-    if (!over_if_exist( src, dst, copy_info ))
+    if ( ! over_if_exist( src, dst, copy_info ) )
       {
       copy_info->skipped_count++;
       return copy_info->abort ? CR_ABORT : CR_SKIP; /* ok */
@@ -688,7 +738,7 @@ int __vfu_dir_erase( const char* target, fsize_t* bytes_freed )
     {
     VString t;
     t.fi( *bytes_freed );
-    str_comma( t );
+    vfu_str_comma( t );
     t = "ERASE: " + t + " bytes freed.";
     say2(t);
     }
@@ -770,7 +820,7 @@ int __vfu_symlink( const char* src, const char* dst, CopyInfo* copy_info )
 {
   errno = 0; /* clear error status */
 
-  if (!over_if_exist( src, dst, copy_info ))
+  if ( ! over_if_exist( src, dst, copy_info ) )
     {
     copy_info->skipped_count++;
     return copy_info->abort ? CR_ABORT : CR_SKIP; /* ok */
@@ -813,6 +863,7 @@ void __copy_calc_totals( CopyInfo &copy_info, int a_one )
 {
   if ( opt.copy_calc_totals )
     {
+    say1( "Calculating files size. Press ESCAPE to cancel calculation." );
     copy_info.files_size = vfu_update_sel_size( a_one );
     copy_info.files_count = a_one ? 1 : sel_count; /* not used */
     copy_info.current_size = 0;
@@ -827,7 +878,7 @@ void __copy_calc_totals( CopyInfo &copy_info, int a_one )
 void vfu_copy_files( int a_one, int a_mode )
 {
   ignore_copy_errors = 0;
-  if ( files_count == 0 )
+  if ( files_list_count() == 0 )
     {
     say1( "No files" );
     return;
@@ -855,10 +906,9 @@ void vfu_copy_files( int a_one, int a_mode )
     target = work_path;
   else
     target = opt.last_copy_path[ a_mode ];
-  const char* cm_mode_str[] = { "COPY", "MOVE", "LINK" };
-  VString str = cm_mode_str[ a_mode ];
+  VString str = CM_DESC[ a_mode ];
   if ( a_one )
-    str = str + " `" + files_list[FLI]->name_ext() + "' to:";
+    str = str + " `" + files_list_get(FLI)->name_ext() + "' to:";
   else
     str += " SELECTED FILES/DIRS to:";
   if ( !vfu_get_dir_name( str, target ) ) return;
@@ -895,7 +945,7 @@ void vfu_copy_files( int a_one, int a_mode )
 
   if ( !copy_info.no_free_check && !copy_info.no_info )
     {
-    fsize_t dev_free = device_free_space( target );
+    fsize_t dev_free = device_avail_space( target );
     if (copy_info.files_size > dev_free )
       {
       vfu_beep();
@@ -910,11 +960,11 @@ void vfu_copy_files( int a_one, int a_mode )
       }
     } /* free space check */
 
-  copy_info.description = "FILE OPERATION: ";
-  copy_info.description += cm_mode_str[ a_mode ];
+  copy_info.description = "FILE ";
+  copy_info.description += CM_DESC[ a_mode ];
   copy_info.description += ": ";
   sprintf( t, "%.0f", copy_info.files_size );
-  str_comma( t );
+  vfu_str_comma( t );
   copy_info.description += t;
   copy_info.description += " bytes.";
 
@@ -928,10 +978,10 @@ void vfu_copy_files( int a_one, int a_mode )
     }
 
   int z;
-  for ( z = 0; z < files_count; z++ )
+  for ( z = 0; z < files_list_count(); z++ )
     {
     if ( vfu_break_op() ) break; /* cancel operation */
-    TF *fi = files_list[z];
+    TF *fi = files_list_get(z);
 
     if ( a_one && z != FLI ) continue; /* if one and not current -- skip */
     if ( !a_one && !fi->sel ) continue; /* if not one and not selected -- skip */
@@ -954,10 +1004,7 @@ void vfu_copy_files( int a_one, int a_mode )
     if ( r == 0 )
       {
       if ( a_mode == CM_MOVE )
-        {
-        delete fi;
-        files_list[z] = NULL;
-        }
+        files_list_del(z);
       }
     else if ( r != CR_SKIP && r != CR_ABORT )
       {
@@ -971,7 +1018,7 @@ void vfu_copy_files( int a_one, int a_mode )
     } /* for files_list[] */
 
   if ( a_mode == CM_MOVE )
-    vfu_pack_files_list();
+    files_list_pack();
   update_status();
   do_draw = 2;
 
@@ -984,15 +1031,15 @@ void vfu_copy_files( int a_one, int a_mode )
   if ( copy_info.current_size > 0 )
     { /* i.e. only if there *are* some bytes copied :) */
     str.fi( copy_info.current_size );
-    str_comma( str );
-    str = copy_info.description + " DONE: " + str + " bytes copied.";
+    vfu_str_comma( str );
+    str = copy_info.description + " DONE: " + str + " bytes.";
     }
   else
     {
     str = copy_info.description;
     str += " DONE";
     }
-  say2( str );
+  say2( str + " Target free: " + size_str_compact( device_avail_space( target ) ) + " (" + size_str_compact( device_free_space( target ) ) + ")" );
 
   ignore_copy_errors = 0;
 }
@@ -1002,7 +1049,7 @@ void vfu_copy_files( int a_one, int a_mode )
 void vfu_erase_files( int a_one )
 {
   ignore_copy_errors = 0;
-  if ( files_count == 0 )
+  if ( files_list_count() == 0 )
     {
     say1( "No files" );
     return;
@@ -1012,11 +1059,12 @@ void vfu_erase_files( int a_one )
   fsize_t *bytes_freed_ptr = opt.bytes_freed ? &bytes_freed : NULL;
 
   VString str;
+  say1( "Calculating files size to be ERASED! Press ESCAPE to cancel calculation." );
   fsize_t erase_size = vfu_update_sel_size( a_one );
   if ( erase_size != -1 )
     {
     str.fi( erase_size );
-    str_comma( str );
+    vfu_str_comma( str );
     }
   else
     {
@@ -1028,7 +1076,7 @@ void vfu_erase_files( int a_one )
   str = "ERASE: " + str + " bytes in: ( ENTER to confirm, other key to cancel )";
   say1( str );
   if ( a_one )
-    say2( files_list[FLI]->full_name() );
+    say2( files_list_get(FLI)->full_name() );
   else
     say2( "SELECTED FILES/DIRS" );
 
@@ -1038,10 +1086,10 @@ void vfu_erase_files( int a_one )
   if (ch != 13) return;
 
   int z;
-  for ( z = 0; z < files_count; z++ )
+  for ( z = 0; z < files_list_count(); z++ )
     {
     if ( vfu_break_op() ) break; /* cancel operation */
-    TF *fi = files_list[z];
+    TF *fi = files_list_get(z);
 
     if ( a_one && z != FLI ) continue;
     if ( !a_one && !fi->sel ) continue;
@@ -1051,7 +1099,7 @@ void vfu_erase_files( int a_one )
     int r = __vfu_erase( target, bytes_freed_ptr );
 
     if ( r == 0 )
-      { delete fi; files_list[z] = NULL; }
+      files_list_del(z);
     else if ( r != CR_ABORT && !ignore_copy_errors )
       {
       vfu_beep();
@@ -1067,7 +1115,7 @@ void vfu_erase_files( int a_one )
     if ( r == CR_ABORT ) break; /* cancel operation */
     } /* for files_list[] */
 
-  vfu_pack_files_list();
+  files_list_pack();
   update_status();
   do_draw = 2;
 
@@ -1076,7 +1124,7 @@ void vfu_erase_files( int a_one )
   if ( opt.bytes_freed )
     {
     str.fi( bytes_freed );
-    str_comma( str );
+    vfu_str_comma( str );
     str = "ERASE DONE: " + str + " bytes freed.";
     say2( str );
     }
@@ -1101,17 +1149,17 @@ void clipboard_add()
     return;
     }
 
-  Clipboard.undef();
+  clipboard.undef();
   clipboard_copy_info.reset();
 
   VString keep = "1";
 
   int z;
-  for ( z = 0; z < files_count; z++ )
+  for ( z = 0; z < files_list_count(); z++ )
     {
-    TF *fi = files_list[z];
+    TF *fi = files_list_get(z);
     if ( !fi->sel ) continue;
-    Clipboard[ fi->full_name() ] = keep;
+    clipboard[ fi->full_name() ] = keep;
     }
 
   __copy_calc_totals( clipboard_copy_info, 0 );
@@ -1127,11 +1175,11 @@ void clipboard_add()
 
 void clipboard_paste( int mode )
 {
-  VArray va = Clipboard.keys();
+  VArray va = clipboard.keys();
 
-  ASSERT(    mode == CLIPBOARD_COPY
-          || mode == CLIPBOARD_MOVE
-          || mode == CLIPBOARD_SYMLINK );
+  ASSERT(    mode == CM_COPY
+          || mode == CM_MOVE
+          || mode == CM_LINK );
 
   ASSERT( !copy_buff );
   copy_buff = new char[1024*1024];
@@ -1149,11 +1197,11 @@ void clipboard_paste( int mode )
     VString dst = work_path + str_file_name_ext( ps );
 
     int r;
-    if ( mode == CLIPBOARD_SYMLINK )
+    if ( mode == CM_LINK )
       r = __vfu_symlink( ps, dst, &clipboard_copy_info );
-    else if ( mode == CLIPBOARD_MOVE )
+    else if ( mode == CM_MOVE )
       r = __vfu_move( ps, dst, &clipboard_copy_info );
-    else // if ( mode == CLIPBOARD_COPY )
+    else // if ( mode == CM_COPY )
       r = __vfu_copy( ps, dst, &clipboard_copy_info );
     if ( r == 0 ) clipboard_copy_info.ok_count++;
     if ( r != 0 && r != CR_SKIP && r != CR_ABORT )
@@ -1173,11 +1221,11 @@ void clipboard_paste( int mode )
 
   vfu_rescan_files( 0 );
   say( 1, cINFO, "CLIPBOARD: %s: %d of %d files processed ok",
-                 CB_DESC[ mode ],
+                 CM_DESC[ mode ],
                  clipboard_copy_info.ok_count,
                  clipboard_copy_info.files_count );
 
-  if ( mode == CLIPBOARD_MOVE ) clipboard_clear();
+  if ( mode == CM_MOVE ) clipboard_clear();
 }
 
 void clipboard_clear()
@@ -1186,13 +1234,13 @@ void clipboard_clear()
     say( 2, cINFO, "CLIPBOARD: %d files removed", clipboard_copy_info.files_count );
   else
     say( 2, cINFO, "CLIPBOARD: empty" );
-  Clipboard.undef();
+  clipboard.undef();
   clipboard_copy_info.reset();
 }
 
 void clipboard_view()
 {
-  mb = Clipboard.keys();
+  mb = clipboard.keys();
   if( mb.count() == 0 )
     {
     say2( "CLIPBOARD: empty" );
@@ -1214,9 +1262,9 @@ void clipboard_menu( int act )
     mb.push( "V View clipboard" );
 
     VString fcnt = clipboard_copy_info.files_count;
-    str_comma( fcnt );
+    vfu_str_comma( fcnt );
     VString fsize = clipboard_copy_info.files_size;
-    str_comma( fsize );
+    vfu_str_comma( fsize );
     mb.push( "---  " + fcnt + " files, " + fsize + " bytes" );
 
     if ( vfu_menu_box( 5, 5, "File Clipboard " + fcnt + " files, " + fsize + " bytes" ) == -1 ) return;
@@ -1226,9 +1274,9 @@ void clipboard_menu( int act )
   switch( act )
     {
     case 'A': clipboard_add(); break;
-    case 'P': clipboard_paste( CLIPBOARD_COPY ); break;
-    case 'O': clipboard_paste( CLIPBOARD_MOVE ); break;
-    case 'L': clipboard_paste( CLIPBOARD_SYMLINK ); break;
+    case 'P': clipboard_paste( CM_COPY ); break;
+    case 'O': clipboard_paste( CM_MOVE ); break;
+    case 'L': clipboard_paste( CM_LINK ); break;
     case 'E': clipboard_clear(); break;
     case 'V': clipboard_view(); break;
     }
