@@ -1,11 +1,11 @@
-/*
+/****************************************************************************
  *
- * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2012
+ * Copyright (c) 1996-2020 Vladi Belperchinov-Shabanski "Cade" 
  * http://cade.datamax.bg/  <cade@biscom.net> <cade@bis.bg> <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
- */
+ ****************************************************************************/
 
 #include "vfudir.h"
 #include "vfuopt.h"
@@ -19,12 +19,13 @@ VArray size_cache;
 
 /*###########################################################################*/
 
-  void __glob_gdn( const char* a_path, const char* a_fnpattern,
-                   VArray &a_va ) // glob getdirname
+  void __glob_gdn( const char* a_path, const char* a_fnpattern, VArray &a_va, int type = 'D' ) // glob getdirname, type = 'F'ile, 'D'ir, 'A'ny
   {
     VString pat = a_fnpattern;
     pat += "*";
 
+    ASSERT( type == 'F' || type == 'D' || type == 'A' );
+
     a_va.undef();
     DIR *dir;
     dirent *de;
@@ -47,11 +48,17 @@ VArray size_cache;
           {
           VString str; // = a_path;
           str += de->d_name;
-          if ( file_is_dir( a_path + str ) )
+          if( file_is_dir( a_path + str ) )
             {
             str += "/";
-            a_va.push(str);
+            if( type != 'F' )
+              a_va.push(str);
             }
+          else
+            {
+            if( type != 'D' )
+              a_va.push(str);
+            }  
           }
         }
       closedir(dir);
@@ -62,7 +69,7 @@ VArray size_cache;
   FIXME: must call sayX() at the end to clear...
 */
 
-int vfu_get_dir_name( const char *prompt, VString &target, int should_exist )
+int vfu_get_dir_name( const char *prompt, VString &target, int should_exist, int type )
 {
   int res = -1;
   /*
@@ -118,7 +125,7 @@ int vfu_get_dir_name( const char *prompt, VString &target, int should_exist )
       ch = 0;
       }
 
-    if ((ch == 8 || ch == KEY_BACKSPACE) && pos > 0)
+    if ( ( ch == 8 || ch == KEY_BACKSPACE ) && pos > 0 )
       {
       pos--;
       str_del( target, pos, 1 );
@@ -134,7 +141,7 @@ int vfu_get_dir_name( const char *prompt, VString &target, int should_exist )
       pos = z;
       }
     else
-    if ( ch == 9 && str_len( target ) > 0)
+    if ( ch == 9 && str_len( target ) > 0 )
       {
       int z;
       dir_list.undef();
@@ -160,17 +167,21 @@ int vfu_get_dir_name( const char *prompt, VString &target, int should_exist )
         }
       */
 
-      __glob_gdn( dmain, dtail, dir_list );
+      __glob_gdn( dmain, dtail, dir_list, type );
 
       z = dir_list.count()-1;
       if (dir_list.count())
         {
         if ( dir_list.count() > 1)
           {
-          int li; /* counter */
+          int li = 0; /* counter */
+          int ll = 0; /* longest directory entry */
           int xm = 0; /* exact match entry  */
           for ( li = 0; li < dir_list.count(); li++ )
             {
+            int len = strlen( dir_list[li] );
+            if( len > ll )
+              ll = len;
             VString tmp1;
             if( dtail != str_copy( tmp1, dir_list[li], 0, str_len( dtail ) ) )
               continue;
@@ -197,6 +208,8 @@ int vfu_get_dir_name( const char *prompt, VString &target, int should_exist )
             if ( mc != dir_list.count() )
               break;
             mi++;
+            if( mi >= ll )
+              break;
             }
           target.setn( dmain + dir_list[xm], str_len( dmain ) + mi );
           pos = str_len( target );
@@ -261,7 +274,7 @@ int vfu_get_dir_name( const char *prompt, VString &target, int should_exist )
         pos = str_len( target );
       }
     else
-    if (ch >= 32 && ch <= 255 ) // && pos < 70)
+    if ( ch >= 32 && ch <= 255 && ch != KEY_BACKSPACE ) // && pos < 70)
       {
       if (firsthit)
         {
@@ -334,7 +347,7 @@ int vfu_get_dir_name( const char *prompt, VString &target, int should_exist )
   leaveok(stdscr, TRUE);
   #endif
   */
-  if ( res == 1 && str_len( target ) > 0 && should_exist && !dir_exist( target ))
+  if ( res == 1 && str_len( target ) > 0 && should_exist && type == 'D' && !dir_exist( target ))
     {
     vfu_beep();
     int ch = tolower( vfu_ask( "Directory does not exist! Create? "
@@ -363,7 +376,8 @@ int vfu_get_dir_name( const char *prompt, VString &target, int should_exist )
   say2(" ");
   if ( str_len( target ) > 0)
     {
-    str_fix_path( target );
+    if( file_is_dir( target ) )
+      str_fix_path( target );
     vfu_hist_add( HID_GETDIR, target );
     }
 
@@ -387,7 +401,7 @@ void vfu_chdir( const char *a_new_dir )
   else
     {
     target = vfu_hist_get( HID_CHDIR, 0 );
-    if (!vfu_get_dir_name( "ChDir to? (TAB, PageUp, PageDown, ^X, ^A)",
+    if (!vfu_get_dir_name( "ChDir to? (use keys: TAB, PageUp, PageDown, Ctrl+X, Ctrl+A)",
                            target, 0 ))
                            return; /* get_dir_name canceled */
     }
@@ -450,7 +464,7 @@ void vfu_chdir( const char *a_new_dir )
     }
   if (chdir( target ) != 0)
     {
-    sprintf( t, "chdir: %s", target.data() );
+    snprintf( t, sizeof(t), "chdir: %s", target.data() );
     say1( t );
     say2errno();
     return;
@@ -565,7 +579,7 @@ fsize_t __tree_rebuild_process( const char* path )
     if ( strcmp( de->d_name, "." ) == 0 ||
          strcmp( de->d_name, ".." ) == 0 ) continue;
 
-    sprintf(new_name, "%s%s", path, de->d_name);
+    snprintf( new_name, sizeof(new_name), "%s%s", path, de->d_name );
     lstat(new_name, &st);
     int is_link = int(S_ISLNK(st.st_mode));
 
@@ -690,7 +704,7 @@ void tree_draw_item( int page, int index, int hilite )
   if ( sz == "-1" )
     sz = "n/a";
   else
-    str_comma( sz );
+    vfu_str_comma( sz );
   str_pad( sz, 14 );
 
   s1 = sz + " " + s1;
@@ -759,7 +773,7 @@ void tree_draw_pos( ScrollPos &scroll, int opos )
 
   VString sz;
   sz.fi( size_cache_get( str ) );
-  str_comma( sz );
+  vfu_str_comma( sz );
   str_pad( sz, 14 );
   str = sz + " " + str;
 
@@ -795,7 +809,7 @@ void tree_view()
 
   ScrollPos scroll;
   scroll.set_min_max( 0, dir_tree.count()-1 );
-  scroll.set_pagesize( PS+2 );
+  scroll.set_pagesize( FPS + 2 );
   scroll.go( new_pos );
 
   int key = 0;
@@ -910,8 +924,23 @@ void tree_view()
 
 /*###########################################################################*/
 
-#define SIZE_CACHE_OFFSET 12
-#define SIZE_CACHE_OFFSET_CLEAN (12+8+1)
+#define SIZE_CACHE_OFFSET      15
+#define SIZE_CACHE_OFFSET_STR "15"
+#define SIZE_CACHE_OFFSET_CLEAN (SIZE_CACHE_OFFSET+8+1)
+
+void size_cache_load()
+{
+  size_cache.undef();
+  size_cache.fload( filename_size_cache );
+  // removes old-style size cache entries
+  if( size_cache.count() > 0 && ( size_cache[0][SIZE_CACHE_OFFSET] != '|' || size_cache[0][SIZE_CACHE_OFFSET_CLEAN] != '|' ) )
+    size_cache.undef();
+}
+
+void size_cache_save()
+{
+  size_cache.fsave( filename_size_cache );
+}
 
 int size_cache_cmp( const char* s1, const char* s2 )
 {
@@ -928,7 +957,8 @@ VString size_cache_compose_key( const char *s, fsize_t size )
   char s_adler[16];
   char s_size[32];
 
-  sprintf( s_size, "%012.0f", size ); //WARNING: THIS IS SIZE_CACHE_OFFSET
+  // MUST BE SIZE_CACHE_OFFSET!
+  sprintf( s_size, "%0" SIZE_CACHE_OFFSET_STR ".0f", size );
   sprintf( s_adler, "%08X", (unsigned int)str_adler32( ps ) );
 
   VString str;
@@ -997,13 +1027,14 @@ void size_cache_append( const char *s, fsize_t size )
 void size_cache_clean( const char *s )
 {
   VString str = size_cache_compose_key( s, 0 );
+  size_t qc = str_len( str );
   str_trim_left( str, SIZE_CACHE_OFFSET_CLEAN );
   int sl = str_len( str );
   int z = 0;
   while( z < size_cache.count() )
     {
     const char* ps = size_cache[z].data() + SIZE_CACHE_OFFSET_CLEAN;
-    if ( ( strncmp( ps, str, sl ) == 0 && (ps[sl] == '/' || ps[sl] == 0) )
+    if ( ( strlen( size_cache[z] ) >= qc && strncmp( ps, str, sl ) == 0 && (ps[sl] == '/' || ps[sl] == 0) )
          ||
          ( size_cache[z][SIZE_CACHE_OFFSET_CLEAN] != '|' ) )
       size_cache.del( z );
@@ -1108,7 +1139,7 @@ int tree_find( const char *s, VArray *va )
 
 /*###########################################################################*/
 
-fsize_t __dir_size_process( const char* path )
+fsize_t __dir_size_process( const char* path, int mode, dev_t src_dev = 0 )
 {
   if ( vfu_break_op() )
     return -1;
@@ -1126,7 +1157,7 @@ fsize_t __dir_size_process( const char* path )
     if ( strcmp( de->d_name, "." ) == 0 ||
          strcmp( de->d_name, ".." ) == 0 ) continue;
 
-    sprintf(new_name, "%s%s", path, de->d_name);
+    snprintf( new_name, sizeof(new_name), "%s%s", path, de->d_name );
     lstat(new_name, &st);
     int is_link = int(S_ISLNK(st.st_mode));
     memset( &st, 0, sizeof(st) );
@@ -1137,11 +1168,12 @@ fsize_t __dir_size_process( const char* path )
     #endif
     int is_dir = int(S_ISDIR(st.st_mode));
 
-    if ( is_link ) continue; /* skip links */
-    if ( is_dir )
+    if ( is_link && ! ( mode & DIR_SIZE_FOLLOWSYMLINKS ) ) continue; /* skip links */
+    if ( is_dir  )
       {
+      if ( src_dev && src_dev != st.st_dev ) continue;
       strcat( new_name, "/" );
-      fsize_t dir_size = __dir_size_process( new_name );
+      fsize_t dir_size = __dir_size_process( new_name, mode );
       if ( dir_size == -1 )
         {
         closedir(dir);
@@ -1164,14 +1196,22 @@ fsize_t __dir_size_process( const char* path )
   return size;
 }
 
-fsize_t vfu_dir_size( const char *s, int sort )
+fsize_t vfu_dir_size( const char *s, int sort, int mode )
 {
   char t[MAX_PATH];
   expand_path( s, t );
-  size_cache_clean( t );
+  //size_cache_clean( t );
   str_fix_path( t );
   size_cache_clean( t );
-  fsize_t size = __dir_size_process( t );
+
+  int src_dev = 0;
+  if( mode & DIR_SIZE_SAMEDEVONLY )
+    {
+    struct stat st;
+    stat( s, &st);
+    src_dev = st.st_dev;
+    }
+  fsize_t size = __dir_size_process( t, mode, src_dev );
   size_cache_set( t, size, sort );
   return size;
 }
