@@ -1,11 +1,11 @@
-/*
+/****************************************************************************
  *
- * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2012
+ * Copyright (c) 1996-2020 Vladi Belperchinov-Shabanski "Cade" 
  * http://cade.datamax.bg/  <cade@biscom.net> <cade@bis.bg> <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
- */
+ ****************************************************************************/
 
 #ifndef _VFUDIR_H_
 #define _VFUDIR_H_
@@ -16,7 +16,7 @@ extern VArray size_cache;
 
 /*###########################################################################*/
 
-int vfu_get_dir_name( const char *prompt, VString &target, int should_exist = 1 );
+int vfu_get_dir_name( const char *prompt, VString &target, int should_exist = 1, int type = 'D' );
 void vfu_chdir( const char *a_new_dir );
 void vfu_chdir_history();
 
@@ -41,6 +41,8 @@ const char* tree_find( const char *s ); /* return full path by dirname */
 /* return count of found dirnames and stores them to sc */
 int tree_find( const char *s, VArray *va );
 
+void size_cache_load();
+void size_cache_save();
 VString size_cache_compose_key( const char *s, fsize_t size );
 int size_cache_index( const char *s );
 fsize_t size_cache_get( const char *s );
@@ -51,7 +53,11 @@ void size_cache_sort();
 
 /*###########################################################################*/
 
-fsize_t vfu_dir_size( const char *s, int sort = 1 );
+#define DIR_SIZE_NORMAL         0
+#define DIR_SIZE_FOLLOWSYMLINKS 2
+#define DIR_SIZE_SAMEDEVONLY    4
+
+fsize_t vfu_dir_size( const char *s, int sort = 1, int mode = DIR_SIZE_NORMAL );
 
 #endif //_VFUDRI_H_
 
