@@ -1,19 +1,141 @@
-/*
+/****************************************************************************
  *
- * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2012
+ * Copyright (c) 1996-2020 Vladi Belperchinov-Shabanski "Cade" 
  * http://cade.datamax.bg/  <cade@biscom.net> <cade@bis.bg> <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
- */
+ ****************************************************************************/
+
+#include <stdio.h>
 
-#include "vfu.h"
 #include "vfufiles.h"
 #include "vfuopt.h"
 #include "vfuview.h"
 #include "vfumenu.h"
 #include "vfudir.h"
 
+
+/*###########################################################################*/
+
+#define FILES_LIST_BUCKET_SIZE (32*1024)
+
+TF**   files_list       = NULL;
+int    files_list_cnt   = 0;
+int    files_list_size  = 0;
+
+/* index in the files list */
+ScrollPos file_list_index;
+
+void __files_list_resize( int new_size )
+{
+
+  while( new_size < files_list_cnt )
+    {
+    files_list_cnt--;
+    delete files_list[ files_list_cnt ];
+    files_list[ files_list_cnt ] = NULL;
+    }
+  int new_files_list_size = ( int( new_size / FILES_LIST_BUCKET_SIZE ) + 1 ) * FILES_LIST_BUCKET_SIZE;
+
+  if( new_files_list_size == files_list_size ) return;
+
+      
+  TF** new_files_list = new TF*[ new_files_list_size ];
+  memset( new_files_list,          0, sizeof(TF*) * ( new_files_list_size ) );
+  memcpy( new_files_list, files_list, sizeof(TF*) * ( files_list_cnt      ) );
+  delete [] files_list;
+
+  files_list      = new_files_list;
+  files_list_size = new_files_list_size;
+  
+  file_list_index.set_min_max( 0, files_list_cnt - 1 );
+}
+
+int  files_list_count()
+{
+  return files_list_cnt;
+}
+
+int  files_list_is_empty( int pos )
+{
+  ASSERT( pos >= 0 && pos < files_list_cnt );
+  return files_list[pos] == NULL ? 1 : 0;
+}
+
+TF* files_list_get( int pos )
+{
+  ASSERT( pos >= 0 && pos < files_list_cnt );
+  ASSERT( files_list[pos] ); // cannot get empty TF*
+  return files_list[pos];
+}
+
+void files_list_set( int pos, TF* fp )
+{
+  ASSERT( pos >= 0 && pos < files_list_cnt );
+  files_list[pos] = fp;
+}
+
+void files_list_add( TF* fp )
+{
+  __files_list_resize( files_list_cnt + 1 );
+  files_list_cnt++;
+  files_list_set( files_list_cnt - 1, fp );
+}
+
+void files_list_trim()
+{
+  if( files_list_cnt <= 0 ) return;
+  __files_list_resize( files_list_cnt - 1 );
+}
+
+void files_list_del( int pos )
+{
+  ASSERT( pos >= 0 && pos < files_list_cnt );
+  ASSERT( files_list[pos] );
+  delete files_list[pos];
+  files_list[pos] = NULL;
+}
+
+void files_list_clear()
+{
+  __files_list_resize( 0 );
+}
+
+void files_list_pack()
+{
+  int pos  = 0;
+  int next = 0;
+
+  while( pos < files_list_cnt )
+    {
+    if ( files_list[pos] == NULL )
+      {
+      next = pos + 1;
+      while ( next < files_list_cnt && files_list[next] == NULL ) next++;
+      if ( next < files_list_cnt && files_list[next] != NULL )
+        {
+        files_list[pos] = files_list[next];
+        files_list[next] = NULL;
+        }
+      else
+        break;
+      }
+    else
+      pos++;
+    }
+  files_list_cnt = 0;
+  while ( files_list_cnt < files_list_size && files_list[files_list_cnt] ) files_list_cnt++;
+
+  /* update scroll parameters */
+  file_list_index.set_min_max( 0, files_list_cnt - 1 );
+  file_list_index.set_pagesize(  con_max_y() - 7 );
+
+  update_status();
+  vfu_nav_update_pos();
+  do_draw = 2;
+}
+
 /*###########################################################################*/
 
 static char __file_stat_type_buf[3];
@@ -30,7 +152,7 @@ const char* file_type_str( mode_t mode, int is_link )
   if (S_ISDIR (mode)) strcpy(__file_stat_type_buf, "[]"); else // box
   if ((mode & S_IXOTH)||(mode & S_IXGRP)||(mode & S_IXUSR))
                       strcpy(__file_stat_type_buf, "**"); else // * marks executables
-  ;
+  {};
   return __file_stat_type_buf;
 }
 
@@ -56,10 +178,10 @@ void vfu_rescan_files( int a_recursive )
   int savea_count = 0;
   if ( opt.keep_selection && sel_count > 0 )
     {
-    for ( z = 0; z < files_count ; z++)
-      if ( files_list[z]->sel )
+    for ( z = 0; z < files_list_cnt ; z++)
+      if ( files_list_get(z)->sel )
         {
-        savea[ files_list[z]->name() ] = keep;
+        savea[ files_list_get(z)->name() ] = keep;
         savea_count++;
         }
     }
@@ -69,35 +191,27 @@ void vfu_rescan_files( int a_recursive )
   /* restore selection */
   if ( opt.keep_selection && savea_count > 0 )
     {
-    for ( z = 0; z < files_count ; z++ )
-      if ( savea.exists( files_list[z]->name() ) )
-        files_list[z]->sel = 1;
+    for ( z = 0; z < files_list_count() ; z++ )
+      if ( savea.exists( files_list_get(z)->name() ) )
+        files_list_get(z)->sel = 1;
     update_status();
     }
 
   file_list_index.set_page( old_flp );
   file_list_index.set_pos( old_fli );
   vfu_nav_update_pos();
+
+
 }
 
 /*---------------------------------------------------------------------------*/
 
 void vfu_read_files( int a_recursive )
 {
-  say1( "Rescanning files..." );
-
-  int z;
+  say1( "Rescanning files... press ESC to interrupt" );
 
   /* clear files list -- delete all found entries */
-  for ( z = 0; z < MAX_FILES ; z++)
-    if (files_list[z])
-      {
-      delete files_list[z];
-      files_list[z] = NULL;
-      }
-
-  /* vfu_add_file() will need this */
-  files_count = 0;
+  files_list_clear();
 
   /* FIXME: perhaps we could check work_mode here? ... anyway will ASSERT it */
   if ( archive_name != "" )
@@ -122,7 +236,7 @@ void vfu_read_files( int a_recursive )
     }
 
   /* update scroll parameters */
-  file_list_index.set_min_max( 0, files_count - 1 );
+  file_list_index.set_min_max( 0, files_list_cnt - 1 );
   file_list_index.set_pagesize( con_max_y() - 7 );
 
   update_status();
@@ -141,7 +255,7 @@ void vfu_read_files( int a_recursive )
 
 int vfu_add_file( const char* fname, const struct stat *st, int is_link )
 {
-  if ( files_count == MAX_FILES ) return 1;
+
   VString ne = str_file_name_ext( fname );
 
   if ( ne == "."  || ne == ".." ) return 0;
@@ -158,30 +272,40 @@ int vfu_add_file( const char* fname, const struct stat *st, int is_link )
     #endif
     }
 
-  if ( !S_ISDIR( st->st_mode ) ) /* mask is not allowed for dirs */
+  int is_dir = S_ISDIR( st->st_mode );
+  if ( ! is_dir ) /* mask is not allowed for dirs */
     if ( vfu_fmask_match( ne ) ) return 0; /* doesn't match the mask */
   TF *fi = new TF( fname, st, is_link );
-  files_list[files_count] = fi;
-  files_count++;
 
-  /* get dir sizes for directories, not symlinks */
-  if ( work_mode == WM_NORMAL && fi->is_dir() && !fi->is_link() )
-    fi->set_size( size_cache_get( fi->full_name( 1 ) ) );
-  /* get dir sizes for directories, symlinks */
-  if ( work_mode == WM_NORMAL && fi->is_dir() && fi->is_link() )
+  files_list_add( fi );
+
+  /* get dir sizes for directories */
+  if ( work_mode == WM_NORMAL && is_dir )
     {
-    char t[MAX_PATH];
-    expand_path( fi->full_name( 1 ), t );
-    str_fix_path( t );
-    fi->set_size( size_cache_get( t ) );
+    if ( is_link )
+      {
+      /* symlinks */
+      char t[MAX_PATH];
+      expand_path( fi->full_name( 1 ), t );
+      str_fix_path( t );
+      fi->set_size( size_cache_get( t ) );
+      }
+    else
+      {
+      /* not symlinks */
+      fi->set_size( size_cache_get( fi->full_name( 1 ) ) );
+      }
     }
 
   /* show progress ... */
-  if ( files_count % 123 == 0 )
+  if ( files_list_cnt % ( files_list_cnt > 1024 ? 373 : 73 ) == 0 )
     {
-    sprintf( ne, "Rescanning files... (%5d)  ", files_count);
+    VString files_list_cnt_str = files_list_cnt;
+    str_comma( files_list_cnt_str );
+    sprintf( ne, "Rescanning files... [%s] press ESC to interrupt", files_list_cnt_str.data() );
     say1( ne );
     }
+
   return 0;
 }
 
@@ -196,16 +320,16 @@ int __vfu_ftw_add( const char* origin, const char* fname,
   VString str = fname;
   str_trim_left( str, str_len( origin ) );
 
-  return vfu_add_file( str, st, file_is_link( fname ) );
+  return vfu_add_file( str, st, is_link );
 }
 
 void vfu_read_local_files( int a_recursive )
 {
   ftwalk( ".", __vfu_ftw_add, a_recursive ? -1 : 1 );
 
-  if ( opt.auto_mount && files_count == 1 &&
-       ( FNMATCH( files_list[0]->name(), "automount" ) == 0 ||
-         FNMATCH( files_list[0]->name(), ".automount" ) == 0 ) )
+  if ( opt.auto_mount && files_list_count() == 1 &&
+       ( FNMATCH( files_list_get(0)->name(), "automount" ) == 0 ||
+         FNMATCH( files_list_get(0)->name(), ".automount" ) == 0 ) )
    {
    VString tmp_file_name;
    tmp_file_name += tmp_path;
@@ -221,12 +345,10 @@ void vfu_read_local_files( int a_recursive )
    if ( (err = system( str )) == 0)
      {
      //---------------
-     delete files_list[0];
-     files_list[0] = NULL;
+     files_list_trim();
      sel_count = 0;
      sel_size = 0;
      files_size = 0;
-     files_count = 0;
      //---------------
      chdir( work_path );
      ftwalk( ".", __vfu_ftw_add, a_recursive ? -1 : 1 );
@@ -236,7 +358,7 @@ void vfu_read_local_files( int a_recursive )
      char t[128];
      FILE *f = fopen( tmp_file_name, "r" );
      t[0] = 0;
-     fgets( t, 100, f );
+     fgets( t, sizeof(t), f );
      fclose(f);
      str_tr( t, "\n\r", "  " );
      say1( "AutoMount failed! ( press ESC ) reason:" );
@@ -267,7 +389,7 @@ void vfu_read_external_files()
 
     while( str_word( tmp, " \t:;", tmp1 ) )
       {
-      if (access( tmp1, F_OK )) continue;
+      if ( access( tmp1, F_OK ) ) continue;
 
       struct stat st;
       stat( tmp1, &st );
@@ -276,7 +398,7 @@ void vfu_read_external_files()
 
       if ( vfu_add_file( tmp1, &st, file_is_link( tmp1 ) ) )
         {
-        pclose(f);
+        pclose( f );
         external_panelizer = ""; /* reset -- there's no reload on this */
         return;
         }
@@ -313,53 +435,17 @@ int vfu_fmask_match( const char* fname )
 
 /*###########################################################################*/
 
-void vfu_pack_files_list()
-{
-  int pos  = 0;
-  int next = 0;
-
-  while( pos < files_count )
-    {
-    if ( files_list[pos] == NULL )
-      {
-      next = pos + 1;
-      while ( next < files_count && files_list[next] == NULL ) next++;
-      if ( next < files_count && files_list[next] != NULL )
-        {
-        files_list[pos] = files_list[next];
-        files_list[next] = NULL;
-        }
-      else
-        break;
-      }
-    else
-      pos++;
-    }
-  files_count = 0;
-  while ( files_count < MAX_FILES && files_list[files_count] ) files_count++;
-
-  /* update scroll parameters */
-  file_list_index.set_min_max( 0, files_count - 1 );
-  file_list_index.set_pagesize(  con_max_y() - 7 );
-
-  update_status();
-  vfu_nav_update_pos();
-  do_draw = 2;
-}
-
-/*###########################################################################*/
-
 /* this compares Name20 and Name3 and returns second as smaller :) (or so) */
 int namenumcmp( const char* s1, const char* s2 )
 {
-  VRegexp re1( "^(.*)([0123456789]+)(\\.(.*))?$" );
-  VRegexp re2( "^(.*)([0123456789]+)(\\.(.*))?$" );
+  VRegexp re1( "^(.*?)([0123456789]+)(\\.(.*))?$" );
+  VRegexp re2( "^(.*?)([0123456789]+)(\\.(.*))?$" );
   if ( re1.m(s1) && re2.m(s2) )
     {
     VString ss1;
     VString ss2;
     sprintf( ss1, "%020d", atoi(re1[2]) );
-    sprintf( ss2, "%020d", atoi(re1[2]) );
+    sprintf( ss2, "%020d", atoi(re2[2]) );
     ss1 = re1[1] + ss1 + re1[3];
     ss2 = re2[1] + ss2 + re2[3];
     return pathcmp( ss1, ss2 );
@@ -379,10 +465,12 @@ TF *f1 = files_list[nf1];
 int z = 0;
 
 /* keep dirs on top */
-if ( f1->is_dir() && !f2->is_dir()) return -1;
-if (!f1->is_dir() &&  f2->is_dir()) return  1;
+if( opt.sort_top_dirs )
+  {
+  if ( f1->is_dir() && !f2->is_dir()) return -1;
+  if (!f1->is_dir() &&  f2->is_dir()) return  1;
+  }
 
-z = 0;
 if (opt.sort_order == 'U') return 0;
 switch (opt.sort_order)
  {
@@ -429,7 +517,7 @@ switch (opt.sort_order)
  default  : ASSERT( !"Non valid sort order (opt.sort_order)" ); break;
  }
 
-if (z == 0) z = pathcmp(f1->name(), f2->name());
+if ( z == 0 ) z = pathcmp( f1->name(), f2->name() );
 
 ASSERT( opt.sort_direction == 'A' || opt.sort_direction == 'D' );
 if (z)
@@ -442,7 +530,7 @@ return z;
 
 /*---------------------------------------------------------------------------*/
 
-void __vfu_sort(int l, int r)
+void __vfu_sort( int l, int r )
 {
   int i;
   int j;
@@ -457,7 +545,7 @@ void __vfu_sort(int l, int r)
   do
     {
     while (ficmp(i,midf) == -1) i++;
-    while (ficmp(j,midf) == 1) j--;
+    while (ficmp(j,midf) ==  1) j--;
     if (i <= j)
       {
       fi = files_list[i];
@@ -476,14 +564,21 @@ void __vfu_sort(int l, int r)
 
 void vfu_sort_files()
 {
-  if (!files_count) return;
+  if ( ! files_list_cnt ) return;
   VString str = files_list[FLI]->name();
-  __vfu_sort( 0, files_count - 1 );
+
+  VString ss = "Sorting... [";
+  str_add_ch( ss, opt.sort_order );
+  str_add_ch( ss, opt.sort_direction );
+  ss += "]";
+  say1( ss );
+  __vfu_sort( 0, files_list_cnt - 1 );
+
   do_draw = 1;
   if ( str != "" )
     {
     int z = 0;
-    for (z = 0; z < files_count; z++)
+    for (z = 0; z < files_list_cnt; z++)
       if ( str == files_list[z]->name() )
         {
         FGO(z);
@@ -529,14 +624,13 @@ void vfu_arrange_files()
   if (_ord == 'R')
     {
     /* Fisher-Yates shuffle */
-    int i = files_count - 1;
+    int i = files_list_count() - 1;
     while( i >= 0 )
       {
       int j = rand() % ( i + 1 );
-      TF *tmp;
-      tmp = files_list[i];
-      files_list[i] = files_list[j];
-      files_list[j] = tmp;
+      TF *tmp = files_list_get( i );
+      files_list_set( i, files_list_get(j) );
+      files_list_set( j, tmp );
       i--;
       }
     do_draw = 2;
@@ -552,7 +646,6 @@ void vfu_arrange_files()
   opt.sort_order = _ord;
   opt.sort_direction = _rev;
 
-  say1("Sorting...");
   vfu_sort_files();
   say1("");
 }
@@ -561,8 +654,8 @@ void vfu_arrange_files()
 
 void vfu_file_entry_move()
 {
-  char t[128];
-  sprintf( t, "MOVE/REORDER File entry: %s", files_list[FLI]->name() );
+  VString t;
+  t = t + "MOVE/REORDER File entry: " + files_list[FLI]->name();
   say1( t );
   say2( "Use Up/Down Arrows to reorder, ESC,ENTER when done." );
   int key = 0;
