@@ -1,11 +1,11 @@
-/*
+/****************************************************************************
  *
- * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2012
+ * Copyright (c) 1996-2020 Vladi Belperchinov-Shabanski "Cade" 
  * http://cade.datamax.bg/  <cade@biscom.net> <cade@bis.bg> <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
- */
+ ****************************************************************************/
 
 #include "vfu.h"
 #include "vfuopt.h"
@@ -24,6 +24,8 @@ const char *TAGMARKS[] = { ">>", "=>", "->", "", "", " ", "", NULL };
 #else
 const char *TAGMARKS[] = { ">>", "=>", "->", NULL };
 #endif
+const char *SIIEC[] = { "IEC", "SI ", NULL };
+const char *COMMA_TYPES[] = { "'", "`", ",", " ", "_", NULL };
 
 ToggleEntry Toggles[] =
 {
@@ -57,6 +59,8 @@ ToggleEntry Toggles[] =
   { ' ', "Use CWD as target for COPY/MOVE", &(opt.default_copy_to_cwd), NOYES },
   {  0 , "--other--", NULL, NULL },
 /*  { ' ', "Can Zap/Erase READ-ONLY Files?!",       &(opt.zap_ro), NOYES }, ? */
+  { ' ', "Keep directories on top of the list",    &(opt.sort_top_dirs), NOYES },
+  { ' ', "Smart HOME/END keys (only Top Dirs mode)",    &(opt.smart_home_end), NOYES },
   { ' ', "Case insensitive file/dir names matching",       &(opt.no_case_glob), NOYES },
   { 'b', "Allow beep!",       &(opt.allow_beep), NOYES },
   { 's', "Free space check on copy",  &(opt.copy_free_space_check), NOYES },
@@ -66,7 +70,10 @@ ToggleEntry Toggles[] =
   { ' ', "Show user's free space", &(opt.show_user_free), NOYES },
   { ' ', "Calc/Show bytes on copy", &(opt.copy_calc_totals), NOYESPRECOPY },
   { ' ', "Calc/Show bytes freed on erase", &(opt.bytes_freed), NOYES },
-  {  -1, "---", NULL }
+  { ' ', "Prefer GiB in disk usage status", &(opt.use_gib_usage), NOYES },
+  { ' ', "Show file/dir sizes in units", &(opt.use_si_sizes), SIIEC },
+  { ' ', "1000s separator type", &(opt.comma_type), COMMA_TYPES },
+  {  -1, "---", NULL, NULL }
 };
 
 /*---------------------------------------------------------------------------*/
@@ -123,7 +130,7 @@ void vfu_load_dir_colors()
     if ( str_find( str, "35" ) != -1 ) pos = cMAGENTA; else
     if ( str_find( str, "36" ) != -1 ) pos = cCYAN; else
     if ( str_find( str, "37" ) != -1 ) pos = cWHITE; else
-    ;
+    {};
 
     int spc = str_find( str, ' ' );
     if ( spc == -1 || pos == -1 ) continue;
@@ -293,6 +300,10 @@ void vfu_settings_load()
 
   opt.bytes_freed = 1;
 
+  opt.use_si_sizes = 0;
+
+  opt.smart_home_end = 1;
+
   /***** LOAD DEFAULTS END ***/
 
   FILE *fsett;
@@ -420,8 +431,7 @@ void vfu_settings_load()
   if (opt.use_dir_colors) vfu_load_dir_colors();
 //  if (file_load_crc32( filename_size_cache, &size_cache, sizeof(size_cache)))
 //    memset( &size_cache, 0, sizeof(size_cache) );
-  size_cache.undef();
-  size_cache.fload( filename_size_cache );
+  size_cache_load();
 }
 
 /*---------------------------------------------------------------------------*/
@@ -431,7 +441,7 @@ void vfu_settings_save()
   file_save_crc32( filename_opt, &opt, sizeof(opt));
 //  file_save_crc32( filename_size_cache, &size_cache, sizeof(size_cache));
   history.fsave( filename_history );
-  size_cache.fsave( filename_size_cache );
+  size_cache_save();
 }
 
 /*---------------------------------------------------------------------------*/
