@@ -1,11 +1,11 @@
-/*
+/****************************************************************************
  *
- * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2012
+ * Copyright (c) 1996-2020 Vladi Belperchinov-Shabanski "Cade" 
  * http://cade.datamax.bg/  <cade@biscom.net> <cade@bis.bg> <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
- */
+ ****************************************************************************/
 
 #ifndef _SETUP_H_
 #define _SETUP_H_
@@ -18,9 +18,9 @@
  *
  */
 
-#define VFU_VERSION "4.10"
+#define VFU_VERSION "4.21"
 
-#define HEADER "VF/U v" VFU_VERSION " (" __DATE__ ") by (c) Vladi Belperchinov-Shabanski `Cade' 1996-2009"
+#define HEADER "VF/U v" VFU_VERSION " Copyright (c) 1996-2020 Vladi Belperchinov-Shabanski 'Cade'"
 #define CONTACT "<cade@biscom.net> <cade@datamax.bg> [http://cade.datamax.bg]"
 
 #ifdef _TARGET_GO32_
@@ -39,6 +39,11 @@
   #define FILENAME_FFR     "vfu.ffr"
 #endif
 
+// The SYSCONFDIR macro can be used here, which is defined when
+// ../configure --sysconfdir=nnnn is used. It no option is given, it
+// defaults to /usr/local/etc on most systems.
+// The macro has been added to Makefile.am
+//                                    -andy5995 2018-12-29
 #ifndef FILENAME_CONF_GLOBAL0
 #define FILENAME_CONF_GLOBAL0 "/etc/" FILENAME_CONF
 #endif
@@ -48,12 +53,12 @@
 #endif
 
 #ifndef FILENAME_CONF_GLOBAL2
-#define FILENAME_CONF_GLOBAL2 "/usr/lib/vfu/" FILENAME_CONF
+#define FILENAME_CONF_GLOBAL2 "/usr/libexec/vfu/" FILENAME_CONF
 #endif
 
 #define RX_TEMP_LIST "RX_TEMP_LIST"
 
-#define MAX_FILES  128000
+#define MAX_FILES  8388608 // 8 Mi, with 8-byte pointer this acquires 64MiB
 
 /* colors */
 
