@@ -1,11 +1,11 @@
-/*
+/****************************************************************************
  *
- * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2012
+ * Copyright (c) 1996-2020 Vladi Belperchinov-Shabanski "Cade" 
  * http://cade.datamax.bg/  <cade@biscom.net> <cade@bis.bg> <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
- */
+ ****************************************************************************/
 
 #include "vfumenu.h"
 #include "vfucopy.h"
@@ -41,7 +41,6 @@ void vfu_tool_classify()
 {
   /* FIXME: how this will handle files with path in the list? */
   char tmp[MAX_PATH];
-  int tl; // tmplen
   if ( sel_count == 0 )
     {
     say1( "Classify function works only on already selected files..." );
@@ -49,34 +48,33 @@ void vfu_tool_classify()
     }
 
   mb.undef();
-  mb.push("N Name");
-  mb.push("E Ext");
-  mb.push("1 First 1 letter");
-  mb.push("2 First 2 letters");
-  mb.push("3 First 3 letters");
-  mb.push("4 First 4 letters");
-  mb.push("5 First 5 letters");
-  mb.push("6 First 6 letters");
-  mb.push("7 First 7 letters");
-  mb.push("8 First 8 letters");
-  mb.push("9 First 9 letter");
-  if ( vfu_menu_box( 50, 5, "Classify files by") == -1 ) return;
+  mb.push( "N Name           ");
+  mb.push( "E Ext"             );
+  mb.push( "1 First 1 letter "  );
+  mb.push( "2 First 2 letters" );
+  mb.push( "3 First 3 letters" );
+  mb.push( "4 First 4 letters" );
+  mb.push( "5 First 5 letters" );
+  mb.push( "6 First 6 letters" );
+  mb.push( "7 First 7 letters" );
+  mb.push( "8 First 8 letters" );
+  mb.push( "9 First 9 letter"  );
+  if ( vfu_menu_box( 32, 6, "Classify files by") == -1 ) return;
   char ch = menu_box_info.ec;
 
   mb.undef();
   int z;
   int i;
-  for ( z = 0; z < files_count; z++ )
+  for ( z = 0; z < files_list_count(); z++ )
     {
-    if ( files_list[z]->is_dir() ) continue;
-    if ( !files_list[z]->sel  ) continue;
-    __get_classify_str( files_list[z]->name(), ch, tmp );
-//    tl = strlen( tmp );
+    TF *fi = files_list_get(z);
+    if (   fi->is_dir() ) continue;
+    if ( ! fi->sel      ) continue;
+    __get_classify_str( fi->name(), ch, tmp );
     int found = 0;
     for ( i = 0; i < mb.count(); i++ )
-      // if ( pathncmp( tmp, mb[i], tl ) == 0 ) { found = 1; break; }
       if ( pathcmp( tmp, mb[i] ) == 0 ) { found = 1; break; }
-    if (!found) mb.push(tmp);
+    if ( ! found ) mb.push( tmp );
     }
   for ( i = 0; i < mb.count(); i++ )
     {
@@ -93,14 +91,14 @@ void vfu_tool_classify()
       }
     }
   CopyInfo copy_info;
-  copy_info.files_size = sel_size;
+  copy_info.files_size  = sel_size;
   copy_info.files_count = sel_count;
-  for ( z = 0; z < files_count; z++ )
+  for ( z = 0; z < files_list_count(); z++ )
     {
-    TF *fi = files_list[z];
-    if ( fi->is_dir() ) continue;
+    TF *fi = files_list_get(z);
+    if ( fi->is_dir()  ) continue;
     if ( fi->is_link() ) continue;
-    if ( !fi->sel  ) continue;
+    if ( !fi->sel      ) continue;
     __get_classify_str( fi->name(), ch, tmp );
     strcat( tmp, "/" );
     ASSERT( dir_exist( tmp ) );
@@ -120,24 +118,26 @@ void vfu_tool_rename()
   VString new_name;
   VString t;
 
-  if ( files_count < 1 )
+  if ( files_list_count() < 1 )
     { say1( "No files to rename... (Empty directory)" ); return; };
   if ( sel_count < 1 )
     { say1( "No files to rename... (You have to select required files)" ); return; };
 
   mb.undef();
-  mb.push( "1 README.TXT => readme.txt" );
-  mb.push( "2 README.TXT => readme.TXT" );
-  mb.push( "3 README.TXT => README.txt" );
-  mb.push( "4 readme.txt => README.TXT" );
-  mb.push( "5 readme.txt => README.txt" );
-  mb.push( "6 readme.txt => readme.TXT" );
-  mb.push( "_ Replace spaces with _" );
-  mb.push( "Y Simplify name (RTFM)" );
-  mb.push( "S Sequential rename" );
-  mb.push( "W Swap SymLink w.Original" );
-  mb.push( "R Replace S.Link w.Original" );
-  if (vfu_menu_box( 50, 5, "Rename Tools" ) == -1) return;
+  mb.push( "1 README.TXT => readme.txt"    );
+  mb.push( "2 README.TXT => readme.TXT"    );
+  mb.push( "3 README.TXT => README.txt"    );
+  mb.push( "4 readme.txt => README.TXT"    );
+  mb.push( "5 readme.txt => README.txt"    );
+  mb.push( "6 readme.txt => readme.TXT"    );
+  mb.push( "_ Replace spaces with _"       );
+  mb.push( "Y Simplify name (RTFM)"        );
+  mb.push( "S Sequential rename"           );
+  mb.push( "T Prefix with current date+time" );
+  mb.push( "D Prefix with current date"      );
+  mb.push( "W Swap SymLink with Original"     );
+  mb.push( "R Replace SymLink with Original"   );
+  if (vfu_menu_box( 32, 6, "Rename Tools" ) == -1) return;
   switch( menu_box_info.ec )
     {
     case 'S' : vfu_tool_seq_rename(); break;
@@ -152,9 +152,9 @@ void vfu_tool_rename()
     case '_' :
     case 'Y' :
                err = 0;
-               for ( z = 0; z < files_count; z++ )
+               for ( z = 0; z < files_list_count(); z++ )
                  {
-                 TF* fi = files_list[z];
+                 TF* fi = files_list_get(z);
 
                  // if ( fi->is_dir() ) continue; // why not? ;)
                  if ( !fi->sel ) continue;
@@ -211,6 +211,47 @@ void vfu_tool_rename()
                say1( t );
                break;
 
+    case 'T' :
+    case 'D' :
+               char   time_prefix[32];
+               time_t timenow = time( NULL );
+               tm     tmnow;
+               localtime_r( &timenow, &tmnow );
+               if( strftime( time_prefix, sizeof(time_prefix) - 1, menu_box_info.ec == 'T' ? "%Y%m%d_%H%M%S_" : "%Y%m%d_", &tmnow ) <= 0 )
+                 {
+                 t = "Rename error! Cannot constrict date/time prefix: ";
+                 t += strerror( errno );
+                 say1( t );
+                 break;
+                 }
+
+               err = 0;
+               for ( z = 0; z < files_list_count(); z++ )
+                 {
+                 TF* fi = files_list_get(z);
+
+                 // if ( fi->is_dir() ) continue; // why not? ;)
+                 if ( !fi->sel ) continue;
+                 path = str_file_path( fi->name() );
+                 new_name = path + time_prefix + str_file_name_ext( fi->name() );
+
+                 if ( ! file_exist( new_name) )
+                   {
+                   if ( rename( fi->name(), new_name ) == 0) /* FIXME: full name ? */
+                     {
+                     fi->set_name( new_name );
+                     do_draw = 2; /* FIXME: this should be optimized? */
+                     }
+                   else
+                     err++;
+                   }
+                 else
+                   err++;
+                 }
+               sprintf( t, "Rename complete (errors: %d)", err );
+               say1( t );
+               break;
+
     }
 }
 
@@ -242,9 +283,9 @@ void vfu_tool_seq_rename()
 
   int err = 0;
   int z;
-  for ( z = 0; z < files_count; z++ )
+  for ( z = 0; z < files_list_count(); z++ )
     {
-    TF* fi = files_list[z];
+    TF* fi = files_list_get(z);
 
     if ( fi->is_dir() ) continue;
     if ( !fi->sel ) continue;
@@ -275,9 +316,9 @@ void vfu_tool_replace_sym_org( int swap )
 {
   int err = 0;
   int z;
-  for ( z = 0; z < files_count; z++ )
+  for ( z = 0; z < files_list_count(); z++ )
     {
-    TF* fi = files_list[z];
+    TF* fi = files_list_get(z);
 
     if ( fi->is_dir() ) continue; // FIXME: dali?
     if ( !fi->sel ) continue;
@@ -287,7 +328,7 @@ void vfu_tool_replace_sym_org( int swap )
     VString org = vfu_readlink( sym );
 
     if (access( org, F_OK )) { err++; continue; }
-    if (unlink( sym ))       { err++; continue; }
+    if (unlink( sym ))       { err++; continue; } // FIXME: TODO: correct?
     if (rename( org, sym ))  { err++; continue; }
     if (swap)
       {
