@@ -1,14 +1,15 @@
-/*
+/****************************************************************************
  *
- * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2012
+ * Copyright (c) 1996-2020 Vladi Belperchinov-Shabanski "Cade" 
  * http://cade.datamax.bg/  <cade@biscom.net> <cade@bis.bg> <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
- */
+ ****************************************************************************/
 
 #include "vfu.h"
 #include "vfuuti.h"
+#include "vfufiles.h"
 #include "vfumenu.h"
 #include "vfusys.h"
 #include "vfudir.h"
@@ -49,6 +50,7 @@ int full = 0;
 #endif
 
 while( a_line[i] )
+  {
   if ( a_line[i] == '%' )
     {
     switch( a_line[i+1] )
@@ -70,22 +72,22 @@ while( a_line[i] )
                  if ( a_line[i+1] == 'G' ) full = 1;
                  // FIXME: 'one' is used to merge F and G options in the future
                  int z;
-                 for( z = 0; z < files_count; z++ )
+                 for( z = 0; z < files_list_count(); z++ )
                    {
-                   TF *fi = files_list[z];
+                   TF *fi = files_list_get(z);
                    if (  one && z != FLI ) continue; /* if one and not current -- skip */
                    if ( !one && !fi->sel ) continue; /* if not one and not selected -- skip */
                    if( work_mode == WM_ARCHIVE )
                      {
-                     s = files_list[z]->name();
+                     s = files_list_get(z)->name();
                      }
                    else if( full )
                      {
-                     files_list[z]->full_name();
+                     files_list_get(z)->full_name();
                      }
                    else
                      {
-                     s = files_list[z]->name();
+                     s = files_list_get(z)->name();
                      }
                    if( !one )
                      {
@@ -113,11 +115,11 @@ while( a_line[i] )
                  if ( a_line[i+1] == 'e' )
                    s = str_file_name( s );
                  else
-                   s = files_list[FLI]->ext();
+                   s = files_list_get(FLI)->ext();
                  out += s;
                  break;
       case 's' : /* current file size */
-                 sprintf( s, "%.0f", files_list[FLI]->size() );
+                 sprintf( s, "%.0f", files_list_get(FLI)->size() );
                  out += s;
                  break;
       case '?' : /* prompt user for argument */
@@ -176,6 +178,7 @@ while( a_line[i] )
     str_add_ch( out, a_line[i] );
     i++;
     }
+  }  
 
   a_line = out;
   return 0;
@@ -204,9 +207,9 @@ fsize_t vfu_update_sel_size( int one ) // used before copy/move to calc estimate
   fsize_t size = 0;
   int z;
   int need_size_cache_sort = 0;
-  for( z = 0; z < files_count; z++ )
+  for( z = 0; z < files_list_count(); z++ )
     {
-    TF *fi = files_list[z];
+    TF *fi = files_list_get(z);
 
     if ( one && z != FLI ) continue; /* if one and not current -- skip */
     if ( !one && !fi->sel ) continue; /* if not one and not selected -- skip */
@@ -263,28 +266,59 @@ char* time_str_compact( const time_t tim, char* buf )
 {
   ASSERT( buf );
   time_t timenow = time( NULL );
-  strcpy(buf, ctime(&tim));
+  tm     tim_tm;
+  localtime_r( &tim, &tim_tm );
+  const char* tfm;
   if (timenow > tim + 6L * 30L * 24L * 60L * 60L /* old */
       ||
       timenow < tim - 60L * 60L) /* in the future */
-      strcpy (buf + 11, buf + 19);
-  buf[16] = 0;
-  strcpy(buf, buf+4);
-  if (buf[4] == ' ') buf[4] = '0';
+      {
+      tfm = "%b %d  %Y";
+      }
+    else
+      {
+      tfm = "%b %d %H:%M";
+      }  
+  strftime( buf, 16, tfm, &tim_tm );
   return buf;
 }
 
 /*---------------------------------------------------------------------------*/
 
-char* size_str_compact( const fsize_t siz, char* buf )
+VString size_str_compact( const fsize_t siz )
 {
-  if ( siz < 1024 )
-    sprintf( buf, "%.0fb", siz );
-  else if ( siz < 1024*1024 )
-    sprintf( buf, "%.0fk", siz/1024 );
+  char buf[32];
+  const char* size_str;
+  int units_size = opt.use_si_sizes ? 1000 : 1024;
+  
+  if ( siz < units_size )
+    {
+    sprintf( buf, "%.0f", siz );
+    size_str = "   B";
+    }
+  else if ( siz < 1.0 * units_size * units_size )
+    {
+    sprintf( buf, "%.0f", siz/units_size );
+    size_str = opt.use_si_sizes ? " KB " : " KiB";
+    }
+  else if ( siz < 1.0 * units_size * units_size * units_size )
+    {
+    sprintf( buf, "%.0f", siz/( units_size * units_size ) );
+    size_str = opt.use_si_sizes ? " MB " : " MiB";
+    }
+  else if ( siz < 100.0 * units_size * units_size * units_size )
+    {
+    sprintf( buf, "%.3f", siz/( units_size * units_size * units_size ) );
+    size_str = opt.use_si_sizes ? " GB " : " GiB";
+    }
   else
-    sprintf( buf, "%.0fm", siz/(1024*1024) );
-  return buf;
+    {
+    sprintf( buf, "%.0f", siz/( units_size * units_size * units_size ) );
+    size_str = opt.use_si_sizes ? " GB " : " GiB";
+    }
+  vfu_str_comma( buf );  
+  strcat( buf, size_str );
+  return VString( buf );
 }
 
 /*---------------------------------------------------------------------------*/
@@ -449,8 +483,7 @@ int vfu_get_str( const char *prompt, VString& target, int hist_id, int x, int y
 
 /*---------------------------------------------------------------------------*/
 
-/* to fool gcc warning that mktemp() is not safe, I don't really care :/
-   still there is no mkstemp for directories :)) */
+// FIXME: TODO: enew function for tmp dir name etc.
 char vfu_temp_filename[MAX_PATH];
 const char* vfu_temp()
 {
@@ -460,6 +493,18 @@ const char* vfu_temp()
     return vfu_temp_filename;
 }
 
+/*---------------------------------------------------------------------------*/
+
+char* vfu_str_comma( char* target )
+{
+  return str_comma( target, COMMA_TYPES[opt.comma_type][0] );
+}
+
+VString& vfu_str_comma( VString& target )
+{
+  return str_comma( target, COMMA_TYPES[opt.comma_type][0] );
+}
+
 /*###########################################################################*/
 
 /* eof vfuuti.cpp */
