@@ -1,11 +1,11 @@
-/*
+/****************************************************************************
  *
- * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2012
+ * Copyright (c) 1996-2020 Vladi Belperchinov-Shabanski "Cade" 
  * http://cade.datamax.bg/  <cade@biscom.net> <cade@bis.bg> <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
- */
+ ****************************************************************************/
 
 #ifndef _VFUUTI_H_
 #define _VFUUTI_H_
@@ -29,7 +29,10 @@ VString& vfu_expand_mask( VString& mask );
 char* time_str_compact( const time_t tim, char* buf );
 void vfu_beep();
 
-char* size_str_compact( const fsize_t siz, char* buf );
+VString size_str_compact( const fsize_t siz );
+
+char* vfu_str_comma( char* target );
+VString& vfu_str_comma( VString& target );
 
 /*###########################################################################*/
 
