@@ -1,11 +1,11 @@
-/*
+/****************************************************************************
  *
- * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2012
+ * Copyright (c) 1996-2020 Vladi Belperchinov-Shabanski "Cade" 
  * http://cade.datamax.bg/  <cade@biscom.net> <cade@bis.bg> <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
- */
+ ****************************************************************************/
 
 #include "vfu.h"
 #include "vfufiles.h"
@@ -72,19 +72,40 @@ int get_item_color( TF *fi )
 
 /*-----------------------------------------------------------------------*/
 
-VString fsize_fmt( fsize_t fs ) /* return commified number */
+VString fsize_fmt( fsize_t fs, int use_gib ) /* return commified number */
 {
+  int units_size = opt.use_si_sizes ? 1000 : 1024;
+  
+  fsize_t th = use_gib ? 1024*1024*1024 : 99999999999.0;
+  
   VString str;
-  if( fs > 99999999999.0 ) // 99_999_999_999 11 positions + 3 comma = 14 chars
+  if( fs > th ) // 99_999_999_999 11 positions + 3 comma = 14 chars
     {
-    str.fi( int( fs / ( 1024*1024 ) ) );
-    str_comma( str );
-    str += " MiB";
+    fsize_t ns = fs / ( units_size * units_size );
+    if( ns > 99999999.0 || use_gib ) // 99_999_999_MIB 8 positions + 2 commas + units = 14 chars
+      {
+      ns = fs / ( units_size * units_size * units_size );
+      if( ns > 99999 )
+        str.fi( int( ns ) );
+      else
+        sprintf( str, "%.3f", ns );  
+      vfu_str_comma( str );
+      str += opt.use_si_sizes ? " GB " : " GiB";
+      }
+    else
+      {  
+      if( ns > 99999 )
+        str.fi( int( ns ) );
+      else
+        sprintf( str, "%.3f", ns );  
+      vfu_str_comma( str );
+      str += opt.use_si_sizes ? " MB " : " MiB";
+      }
     }
   else
     {
     str.fi( fs );
-    str_comma( str );
+    vfu_str_comma( str );
     }
   return str;
 }
@@ -103,7 +124,7 @@ void show_pos( int curr, int all )
 void vfu_drop_all_views()
 {
   int z = 0;
-  for( z = 0; z < files_count; z++ ) files_list[z]->drop_view();
+  for( z = 0; z < files_list_count(); z++ ) files_list_get(z)->drop_view();
   do_draw += 1;
 }
 
@@ -111,11 +132,10 @@ void vfu_drop_all_views()
 
 void vfu_draw( int n )
 {
-  VFU_CHECK_LIST_POS( n );
-  if ( n < FLP || n > FLP + PS )
+  if ( n < FLP || n > FLP + FPS )
     return; /* we are out of screen -- don't draw */
 
-  TF* fi = files_list[n];
+  TF* fi = files_list_get(n);
 
   int c = fi->color(); /* color to be used */
   VString view = fi->view();
@@ -207,34 +227,29 @@ void vfu_redraw() /* redraw file list and header */
   tag_mark_pos = strlen( t );
   sel_mark_pos = tag_mark_pos + 2;
 
-  spos += sprintf( spos, "  #NAME    %s",
-                   opt.long_name_view ? "( long name view )" : "" );
+  /* spos += */ sprintf( spos, "  #NAME    %s",
+                         opt.long_name_view ? "( long name view )" : "" );
 
   str_pad( t, - con_max_x() );
   str_sleft( t, con_max_x() );
 
   con_out(1,3, t, cHEADER );
-  show_pos( FLI+1, files_count );
+  show_pos( FLI+1, files_list_count() );
 
   int z;
 
-  for ( z = 0; z < PS; z++ )
+  for ( z = 0; z < FPS; z++ )
     {
     ASSERT( FLP + z >= 0 );
-    if ( FLP + z >= files_count )
+    if ( FLP + z >= files_list_count() || files_list_is_empty( FLP + z ) )
       {
-      con_out( 1, z+4, "~", cPLAIN );
-      con_ce( cPLAIN );
-      }
-    else if ( files_list[FLP+z] == NULL )  /* FIXME: if NULL?! */
-      {
-      con_out( 1, z+4, "~", cPLAIN );
+      con_out( 1, z + 4, "~", cPLAIN );
       con_ce( cPLAIN );
       }
     else
       vfu_draw( FLP + z );
     }
-  if ( files_count <= 0 )
+  if ( files_list_count() <= 0 )
     con_out( ( con_max_x() - 20 ) / 2, 10, " *** No files found *** ", cHEADER);
 
 }
@@ -250,43 +265,49 @@ void vfu_redraw_status() /* redraw bottom status, total,free,selected... */
   /* first line here */
   s1  = "Select:";
   tmp = sel_count;
-  str_comma(tmp);
-  str_pad(tmp,14);
+  vfu_str_comma(tmp);
+  str_pad(tmp,15);
   s1 += tmp;
 
   s1 += "  Free: ";
-  tmp = fsize_fmt( fs_free );
-  str_pad(tmp,14);
+  //tmp = size_str_compact( fs_free );
+  tmp = fsize_fmt( fs_free, opt.use_gib_usage );
+  str_pad( tmp, 14 );
   s1 += tmp;
   if (fs_total == 0 || fs_free > fs_total)
     tmp = "  n/a%";
   else
-    sprintf( 64, tmp, "%5.1f%%", (double)100 * ((double)fs_free / (double)fs_total));
+    sprintf( 64, tmp, "%4.1f%%", (double)100 * ((double)fs_free / (double)fs_total));
 
-  s1 += "  " + tmp + "  FSize:";
+  s1 += "  " + tmp + " FSize:";
+  //tmp = size_str_compact( files_size );
   tmp = fsize_fmt( files_size );
-  str_pad(tmp,14);
+  str_pad( tmp, 15 );
   s1 += tmp;
   if (fs_total == 0 || files_size > fs_total)
     tmp = " n/a%";
   else
-    sprintf(tmp,"%4.1f%%", (double)100 * ((double)files_size / (double)fs_total));
+    sprintf( 64, tmp,"%4.1f%%", (double)100 * ((double)files_size / (double)fs_total));
   s1 += " " + tmp;
 
   /* second line here */
   s2  = "S.Size:";
-
+  //tmp = size_str_compact( sel_size );
   tmp = fsize_fmt( sel_size );
-  str_pad(tmp,14);
+  str_pad(tmp,15);
   s2 += tmp;
+  
   s2 += "  Total:";
-  tmp = fsize_fmt( fs_total );
-  str_pad(tmp,14);
+  //tmp = size_str_compact( fs_total );
+  tmp = fsize_fmt( fs_total, opt.use_gib_usage );
+  str_pad( tmp, 14 );
   s2 += tmp;
 
   tmp = fs_block_size; str_pad( tmp,5 ); s2 += " [" + tmp + "]";
-  sprintf( tmp,"  %s.%s@%s ", user_id_str.data(), group_id_str.data(),
-                              host_name_str.data() );
+  sprintf( tmp,  " %s.%s@%s ", 
+                 user_id_str.data(), 
+                 group_id_str.data(),
+                 host_name_str.data() );
   s2 += tmp;
 
   str_pad( s1, - con_max_x() );
@@ -300,7 +321,7 @@ void vfu_redraw_status() /* redraw bottom status, total,free,selected... */
 
 void vfu_nav_up()
 {
-  if ( files_count == 0) return;
+  if ( files_list_count() == 0) return;
   if ( FLI == 0 ) return;
 
   int old_flp = FLP;
@@ -318,8 +339,8 @@ void vfu_nav_up()
 
 void vfu_nav_down()
 {
-  if ( files_count == 0 ) return;
-  if ( FLI == files_count - 1 ) return;
+  if ( files_list_count() == 0 ) return;
+  if ( FLI == files_list_count() - 1 ) return;
 
   int old_flp = FLP;
   file_list_index.down();
@@ -336,7 +357,7 @@ void vfu_nav_down()
 
 void vfu_nav_ppage()
 {
-if ( files_count == 0 ) return;
+if ( files_list_count() == 0 ) return;
 if ( FLP == 0 && FLI == 0 ) return;
 
   int old_fli = FLI;
@@ -355,8 +376,8 @@ if ( FLP == 0 && FLI == 0 ) return;
 
 void vfu_nav_npage()
 {
-if ( files_count == 0 ) return;
-if ( FLP >= files_count - PS && FLI == files_count - 1 ) return;
+  if ( files_list_count() == 0 ) return;
+  if ( FLP >= files_list_count() - FPS && FLI == files_list_count() - 1 ) return;
 
   int old_fli = FLI;
   int old_flp = FLP;
@@ -374,30 +395,70 @@ if ( FLP >= files_count - PS && FLI == files_count - 1 ) return;
 
 void vfu_nav_home()
 {
-  if ( files_count == 0 ) return;
+  if ( files_list_count() == 0 ) return;
+  ASSERT( FLI >= 0 && FLI <= files_list_count() - 1 );
+  if ( opt.sort_top_dirs && opt.smart_home_end && FLI == 0 ) 
+    {
+    int z = 0;
+    while( z < files_list_count() )
+      {
+      TF *fi = files_list_get(z);
+      if( ! fi->is_dir() ) break;
+      z++;
+      }
+    if( z < files_list_count() )
+      FGO( z );
+    }
+  else
+    FGO( 0 );
+  vfu_nav_update_pos();
+  do_draw = 1;
+/*
+  if ( files_list_count() == 0 ) return;
   if ( FLI == 0 ) return;
   FGO( 0 );
   vfu_nav_update_pos();
   do_draw = 1;
+*/  
 }
 
 /*-----------------------------------------------------------------------*/
 
 void vfu_nav_end()
 {
-  if ( files_count == 0 ) return;
-  if ( FLI >= files_count - 1 ) return;
-  FGO( files_count - 1 );
+  if ( files_list_count() == 0 ) return;
+  ASSERT( FLI >= 0 && FLI <= files_list_count() - 1 );
+  if ( opt.sort_top_dirs && opt.smart_home_end && FLI == files_list_count() - 1 )
+    {
+    int z = FLI;
+    while( z >= 0 )
+      {
+      TF *fi = files_list_get(z);
+      if( fi->is_dir() ) break;
+      z--;
+      }
+    if( z >= 0 )
+      FGO( z );
+    }
+  else
+    FGO( files_list_count() - 1 );
   vfu_nav_update_pos();
   do_draw = 1;
+/*
+  if ( files_list_count() == 0 ) return;
+  if ( FLI >= files_list_count() - 1 ) return;
+  FGO( files_list_count() - 1 );
+  vfu_nav_update_pos();
+  do_draw = 1;
+*/
 }
 
 /*-----------------------------------------------------------------------*/
 
 void vfu_nav_select()
 {
-  if ( files_count == 0 ) return;
-  TF *fi = files_list[FLI];
+  if ( files_list_count() == 0 ) return;
+  TF *fi = files_list_get(FLI);
 
   fsize_t f = fi->size();
   if ( f < 0 ) f = 0; /* dirs w/o size i.e. -1 */
@@ -425,10 +486,10 @@ void vfu_nav_select()
 
 void vfu_nav_update_pos()
 {
- ASSERT( files_count >= 0 );
+ ASSERT( files_list_count() >= 0 );
  if ( FLI < 0 ) FGO( 0 );
- if ( files_count == 0 ) FGO( 0 );
- if ( files_count > 0 && FLI > files_count - 1 ) FGO( files_count - 1 );
+ if ( files_list_count() == 0 ) FGO( 0 );
+ if ( files_list_count() > 0 && FLI > files_list_count() - 1 ) FGO( files_list_count() - 1 );
 }
 
 /* eof vfuview.cpp */
