@@ -1,11 +1,11 @@
-/*
+/****************************************************************************
  *
- * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2012
+ * Copyright (c) 1996-2020 Vladi Belperchinov-Shabanski "Cade" 
  * http://cade.datamax.bg/  <cade@biscom.net> <cade@bis.bg> <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
- */
+ ****************************************************************************/
 
 #ifndef _VFUVIEW_H_
 #define _VFUVIEW_H_
@@ -16,7 +16,7 @@ extern int tag_mark_pos;
 extern int sel_mark_pos;
 
 int get_item_color( TF* fi );
-VString fsize_fmt( fsize_t fs ); /* return commified number */
+VString fsize_fmt( fsize_t fs, int use_gib = 0 ); /* return commified number */
 
 void show_pos( int curr, int all );
 
