@@ -1,11 +1,11 @@
-/*
+/****************************************************************************
  *
- *  (c) Vladi Belperchinov-Shabanski "Cade" 1996-2012
- *  http://cade.datamax.bg/  <cade@biscom.net> <cade@bis.bg> <cade@datamax.bg>
+ * Copyright (c) 1996-2020 Vladi Belperchinov-Shabanski "Cade" 
+ * http://cade.datamax.bg/  <cade@biscom.net> <cade@bis.bg> <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
- */
+ ****************************************************************************/
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -130,14 +130,14 @@
 
   void AnsiPuts( const char *s )
   {
-    printf( s );
+    puts( s );
   }
 
   void AnsiPuts( const char *s, int attr )
   {
     int _ta = a_ta;
     AnsiTA( attr );
-    printf( s );
+    puts( s );
     AnsiTA( _ta  );
   }
 
