@@ -1,11 +1,11 @@
-/*
+/****************************************************************************
  *
- *  (c) Vladi Belperchinov-Shabanski "Cade" 1996-2012
- *  http://cade.datamax.bg/  <cade@biscom.net> <cade@bis.bg> <cade@datamax.bg>
+ * Copyright (c) 1996-2020 Vladi Belperchinov-Shabanski "Cade" 
+ * http://cade.datamax.bg/  <cade@biscom.net> <cade@bis.bg> <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
- */
+ ****************************************************************************/
 
 #include "form_in.h"
 #include "scroll.h"
@@ -53,7 +53,7 @@ int TextInput( int x, int y, const char *prompt, int maxlen, int fieldlen, VStri
       }
     con_xy( x + scroll.pos() - scroll.page() + 1 , y );
     ch = con_getch();
-    if( ch >= 32 && ch <= 255 && str_len(str) < maxlen - 1 )
+    if( ch >= 32 && ch <= 255 && ch != KEY_BACKSPACE && str_len(str) < maxlen - 1 )
       {
       if (firsthit)
         {
@@ -74,7 +74,6 @@ int TextInput( int x, int y, const char *prompt, int maxlen, int fieldlen, VStri
       firsthit = 0;
       }
 
-
     if( ch == 27 )
       {
       res = 0;
