@@ -1,11 +1,11 @@
-/*
+/****************************************************************************
  *
- *  (c) Vladi Belperchinov-Shabanski "Cade" 1996-2012
- *  http://cade.datamax.bg/  <cade@biscom.net> <cade@bis.bg> <cade@datamax.bg>
+ * Copyright (c) 1996-2020 Vladi Belperchinov-Shabanski "Cade" 
+ * http://cade.datamax.bg/  <cade@biscom.net> <cade@bis.bg> <cade@datamax.bg>
  *
- * SEE `README',LICENSE' OR COPYING' FILE FOR LICENSE AND OTHER DETAILS!
+ * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
- */
+ ****************************************************************************/
 
 #include "vsuti.h"
 #include "vstring.h"
@@ -323,7 +323,8 @@ int make_path( const char *s, long mode )
 char* expand_path( const char *src, char *dest )
 {
   #ifdef _TARGET_UNIX_
-  realpath( src, dest);
+  if (!realpath( src, dest))
+    strcpy( dest, src );
   #endif
 
   #ifdef _TARGET_GO32_
